﻿using UnityEngine;
using UnityEngine.UI;

namespace Whitebrim.ShowColliders
{
    class ColliderCanvas : MonoBehaviour
    {
        //enum ToggleType
        //{
        //    boxCollider,
        //    sphereCollider,
        //    capsuleCollider,
        //    boxTrigger,
        //    sphereTrigger,
        //    capsuleTrigger
        //}

        static Color transparent = new Color(0, 0, 0, 0);

        private GameObject canvas, view;
        private Image sample1, sample2;
        private Button close, submit, reset;
        private TMPro.TMP_InputField colliderInput, triggerInput;
        private Toggle boxColliderToggle, boxTriggerToggle, sphereColliderToggle, sphereTriggerToggle, capsuleColliderToggle, capsuleTriggerToggle;

        public void Awake()
        {
            canvas = gameObject;
            view = transform.FindChildRecursivly("View").gameObject;
            sample1 = transform.FindChildRecursivly("Sample 1").GetComponent<Image>();
            sample2 = transform.FindChildRecursivly("Sample 2").GetComponent<Image>();
            close = transform.FindChildRecursivly("Escape").GetComponent<Button>();
            submit = transform.FindChildRecursivly("Submit").GetComponent<Button>();
            reset = transform.FindChildRecursivly("Reset").GetComponent<Button>();
            triggerInput = transform.FindChildRecursivly("Trigger Input").GetComponent<TMPro.TMP_InputField>();
            colliderInput = transform.FindChildRecursivly("Collider Input").GetComponent<TMPro.TMP_InputField>();
            boxColliderToggle = transform.FindChildRecursivly("Box Colliders Toggle").GetComponent<Toggle>();
            sphereColliderToggle = transform.FindChildRecursivly("Sphere Colliders Toggle").GetComponent<Toggle>();
            capsuleColliderToggle = transform.FindChildRecursivly("Capsule Colliders Toggle").GetComponent<Toggle>();
            boxTriggerToggle = transform.FindChildRecursivly("Box Triggers Toggle").GetComponent<Toggle>();
            sphereTriggerToggle = transform.FindChildRecursivly("Sphere Triggers Toggle").GetComponent<Toggle>();
            capsuleTriggerToggle = transform.FindChildRecursivly("Capsule Triggers Toggle").GetComponent<Toggle>();

            Init();
        }

        public void OnEnable()
        {
            RAPI.ToggleCursor(true);
            colliderInput.text = PlayerPrefs.GetString("Whitebrim.ShowColliders.colliderColor", "98F592");
            triggerInput.text = PlayerPrefs.GetString("Whitebrim.ShowColliders.triggerColor", "F05B6C");
            boxColliderToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.boxCollider", 0) == 1;
            sphereColliderToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.sphereCollider", 0) == 1;
            capsuleColliderToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.capsuleCollider", 0) == 1;
            boxTriggerToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.boxTrigger", 0) == 1;
            sphereTriggerToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.sphereTrigger", 0) == 1;
            capsuleTriggerToggle.isOn = PlayerPrefs.GetInt("Whitebrim.ShowColliders.capsuleTrigger", 0) == 1;
        }

        public void OnDisable()
        {
            RAPI.ToggleCursor(false);
        }

        private void Init()
        {
            close.onClick.RemoveAllListeners();
            close.onClick.AddListener(() => Close());
            submit.onClick.RemoveAllListeners();
            submit.onClick.AddListener(() => Submit());
            reset.onClick.RemoveAllListeners();
            reset.onClick.AddListener(() => Default());
            colliderInput.onValueChanged.RemoveAllListeners();
            colliderInput.onValueChanged.AddListener((string s) => OnColliderValueChanged(s));
            triggerInput.onValueChanged.RemoveAllListeners();
            triggerInput.onValueChanged.AddListener((string s) => OnTriggerValueChanged(s));
            //boxColliderToggle.onValueChanged.RemoveAllListeners();
            //boxColliderToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.boxCollider, b));
            //sphereColliderToggle.onValueChanged.RemoveAllListeners();
            //sphereColliderToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.sphereCollider, b));
            //capsuleColliderToggle.onValueChanged.RemoveAllListeners();
            //capsuleColliderToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.capsuleCollider, b));
            //boxTriggerToggle.onValueChanged.RemoveAllListeners();
            //boxTriggerToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.boxTrigger, b));
            //sphereTriggerToggle.onValueChanged.RemoveAllListeners();
            //sphereTriggerToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.sphereTrigger, b));
            //capsuleTriggerToggle.onValueChanged.RemoveAllListeners();
            //capsuleTriggerToggle.onValueChanged.AddListener((bool b) => OnToggle(ToggleType.capsuleTrigger, b));
        }

        private void Save()
        {
            PlayerPrefs.SetString("Whitebrim.ShowColliders.colliderColor", colliderInput.text);
            PlayerPrefs.SetString("Whitebrim.ShowColliders.triggerColor", triggerInput.text);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.boxCollider", boxColliderToggle.isOn ? 1 : 0);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.sphereCollider", sphereColliderToggle.isOn ? 1 : 0);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.capsuleCollider", capsuleColliderToggle.isOn ? 1 : 0);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.boxTrigger", boxTriggerToggle.isOn ? 1 : 0);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.sphereTrigger", sphereTriggerToggle.isOn ? 1 : 0);
            PlayerPrefs.SetInt("Whitebrim.ShowColliders.capsuleTrigger", capsuleTriggerToggle.isOn ? 1 : 0);
        }

        public void Submit()
        {
            canvas.SetActive(false);
            Save();
            ShowColliders.ChangeSettings(sample1.color, sample2.color, boxColliderToggle.isOn, sphereColliderToggle.isOn, capsuleColliderToggle.isOn,
                boxTriggerToggle.isOn, sphereTriggerToggle.isOn, capsuleTriggerToggle.isOn);
        }

        private void Default()
        {
            colliderInput.text = "98F592";
            triggerInput.text = "F05B6C";
            boxColliderToggle.isOn = false;
            sphereColliderToggle.isOn = false;
            capsuleColliderToggle.isOn = false;
            boxTriggerToggle.isOn = false;
            sphereTriggerToggle.isOn = false;
            capsuleTriggerToggle.isOn = false;
        }

        public void Close()
        {
            canvas.SetActive(false);
        }

        public void OnColliderValueChanged(string newValue)
        {
            Color newColor;
            if (ColorUtility.TryParseHtmlString("#" + newValue, out newColor))
                sample1.color = newColor;
            else
                sample1.color = transparent;
        }

        public void OnTriggerValueChanged(string newValue)
        {
            Color newColor;
            if (ColorUtility.TryParseHtmlString("#" + newValue, out newColor))
                sample2.color = newColor;
            else
                sample2.color = transparent;
        }

        //public void OnToggle(ToggleType toggle, bool value)
        //{
        //}
    }
}
