﻿using System.Collections.Generic;
using UnityEngine;

namespace Battlehub.RTCommon
{
    [DefaultExecutionOrder(-50)]
    public class GLRenderer : MonoBehaviour
    {
        private static GLRenderer m_instance;

        public static GLRenderer Instance
        {
            get { return m_instance; }
        }

        private List<IGL> m_renderObjects;
        public void Add(IGL gl)
        {
            if (m_renderObjects.Contains(gl))
            {
                return;
            }

            m_renderObjects.Add(gl);
        }

        public void Remove(IGL gl)
        {
            m_renderObjects.Remove(gl);
        }

        private void Awake()
        {

            m_instance = this;

            m_renderObjects = new List<IGL>();

        }

        private void OnDestroy()
        {
            if (m_instance == this)
            {
                m_instance = null;
            }
        }

        public void Draw(int cullingMask)
        {
            if (m_renderObjects == null)
            {
                return;
            }

            GL.PushMatrix();
            try
            {
                for (int i = 0; i < m_renderObjects.Count; ++i)
                {
                    IGL line = m_renderObjects[i];
                    line.Draw(cullingMask);
                }
            }
            finally
            {
                GL.PopMatrix();
            }
        }
    }
}
