﻿using Battlehub.RTCommon;
using UnityEngine;

namespace Battlehub.RTGizmos
{
    [DefaultExecutionOrder(-50)]
    public abstract class BaseGizmo : RTEComponent, IGL
    {
        public float GridSize = 1.0f;
        public Color LineColor = new Color(0x98 / 255f, 0xF5 / 255f, 0x92 / 255f);
        public Color SelectionColor = new Color(1.0f, 1.0f, 0, 1.0f);
        public Camera SceneCamera;


        public Transform Target;

        private Vector3 m_prevPoint;
        private Vector3 m_normal;

        private void Start()
        {

            if (SceneCamera == null)
            {
                SceneCamera = Camera.main;
            }

            if (Target == null)
            {
                Target = transform;
            }


            if (GLRenderer.Instance == null)
            {
                GameObject glRenderer = new GameObject();
                glRenderer.name = "GLRenderer";
                glRenderer.AddComponent<GLRenderer>();
            }

            if (SceneCamera != null)
            {
                if (!SceneCamera.GetComponent<GLCamera>())
                {
                    SceneCamera.gameObject.AddComponent<GLCamera>();
                }
            }

            if (GLRenderer.Instance != null)
            {
                GLRenderer.Instance.Add(this);
            }

            StartOverride();
        }

        private void OnEnable()
        {
            if (GLRenderer.Instance != null)
            {
                GLRenderer.Instance.Add(this);
            }

            OnEnableOverride();
        }

        private void OnDisable()
        {
            if (GLRenderer.Instance != null)
            {
                GLRenderer.Instance.Remove(this);
            }

            OnDisableOverride();
        }

        private void Update()
        {
            UpdateOverride();
        }

        protected override void AwakeOverride()
        {
            base.AwakeOverride();


        }

        protected override void OnDestroyOverride()
        {
            base.OnDestroyOverride();

            if (GLRenderer.Instance != null)
            {
                GLRenderer.Instance.Remove(this);
            }
        }

        protected virtual void StartOverride()
        {

        }

        protected virtual void OnEnableOverride()
        {

        }

        protected virtual void OnDisableOverride()
        {

        }

        protected virtual void UpdateOverride()
        {

        }

        protected virtual void BeginRecordOverride()
        {

        }

        protected virtual void EndRecordOverride()
        {

        }

        void IGL.Draw(int cullingMask)
        {

            if (Target == null)
            {
                return;
            }

            DrawOverride();
        }

        protected virtual void DrawOverride()
        {

        }

        protected virtual bool HitOverride(int index, Vector3 vertex, Vector3 normal)
        {
            return true;
        }
    }
}
