﻿using System;
using UnityEngine;

namespace Battlehub.RTGizmos
{
    public abstract class CapsuleGizmo : BaseGizmo
    {
        protected abstract Vector3 Center
        {
            get;
            set;
        }
        protected abstract float Radius
        {
            get;
            set;
        }
        protected abstract float Height
        {
            get;
            set;
        }
        protected abstract int Direction
        {
            get;
            set;
        }

        protected override void DrawOverride()
        {
            base.DrawOverride();


            float hs = GetMaxHorizontalScale(true);
            Vector3 scale = GetHandlesScale(true);

            RuntimeGizmos.DrawWireCapsuleGL(Direction,

                GetHeight(),

                Radius, Target.TransformPoint(Center), Target.rotation,

                new Vector3(hs, hs, hs), LineColor);
        }

        private float GetHeight()
        {
            float s;
            float hs = GetMaxHorizontalScale(true);
            if (Direction == 0)
            {
                s = Target.lossyScale.x;
            }
            else if (Direction == 1)
            {
                s = Target.lossyScale.y;
            }
            else
            {
                s = Target.lossyScale.z;
            }

            return Height * s / hs;
        }

        private Vector3 GetHandlesScale(bool multiplyByTargetScale)
        {
            float x;
            float y;
            float z;
            float hs = GetMaxHorizontalScale(multiplyByTargetScale);
            if (Direction == 0)
            {
                x = GetHandlesHeight(multiplyByTargetScale);
                y = hs * Radius;
                z = hs * Radius;
            }
            else if (Direction == 1)
            {
                x = hs * Radius;
                y = GetHandlesHeight(multiplyByTargetScale);
                z = hs * Radius;
            }
            else
            {
                x = hs * Radius;
                y = hs * Radius;
                z = GetHandlesHeight(multiplyByTargetScale);
            }

            const float min = 0.001f;
            if (x < min && x > -min)
            {
                x = 0.001f;
            }
            if (y < min && y > -min)
            {
                y = 0.001f;
            }
            if (z < min && z > -min)
            {
                z = 0.001f;
            }
            return new Vector3(x, y, z);
        }

        private float GetHandlesHeight(bool multiplyByTargetScale)
        {
            if (!multiplyByTargetScale)
            {
                return MaxAbs(Height / 2, Radius);
            }

            if (Direction == 0)
            {
                return MaxAbs(Target.lossyScale.x * Height / 2, Radius * GetMaxHorizontalScale(multiplyByTargetScale));
            }
            else if (Direction == 1)
            {
                return MaxAbs(Target.lossyScale.y * Height / 2, Radius * GetMaxHorizontalScale(multiplyByTargetScale));
            }

            return MaxAbs(Target.lossyScale.z * Height / 2, Radius * GetMaxHorizontalScale(multiplyByTargetScale));

        }

        private float GetMaxHorizontalScale(bool multiplyByTargetScale)
        {
            if (!multiplyByTargetScale)
            {
                return 1;
            }

            if (Direction == 0)
            {
                return MaxAbs(Target.lossyScale.y, Target.lossyScale.z);
            }
            else if (Direction == 1)
            {
                return MaxAbs(Target.lossyScale.x, Target.lossyScale.z);
            }

            return MaxAbs(Target.lossyScale.x, Target.lossyScale.y);
        }

        private float MaxAbs(float a, float b)
        {
            if (Math.Abs(a) > Math.Abs(b))
            {
                return a;
            }
            return b;
        }
    }
}
