﻿using Battlehub.RTCommon;
using Battlehub.RTGizmos;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace Whitebrim.ShowColliders
{
    public class ShowColliders : Mod
    {
        public static ShowColliders Instance;
        public static Material colliderShader;
        AssetBundle asset;
        GameObject colliderCanvas;


        public static Color normalColor = new Color(0x98 / 255f, 0xF5 / 255f, 0x92 / 255f);
        public static Color triggerColor = new Color(0xF0 / 255f, 0x5B / 255f, 0x6C / 255f);

        public static List<string> banList = new List<string>() { "ColliderPrefabEnabler (Clone)", "IncapacitatedCollider", "BuoyancyPoint" };

        public void Awake()
        {
            Instance = this;

            StartCoroutine(LoadAssets());
            
            Debug.Log("ShowColliders has been loaded!");
        }

        private IEnumerator LoadAssets()
        {
            AssetBundleCreateRequest bundleRequest = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("ModData/showcolliders.assets"));
            yield return bundleRequest;
            asset = bundleRequest.assetBundle;

            var assetRequest = asset.LoadAssetAsync<Shader>("VertexColor");
            yield return assetRequest;
            colliderShader = new Material(assetRequest.asset as Shader);
            Init();
        }

        private void Init()
        {
            RuntimeGizmos.Init();
        }

        [ConsoleCommand(name: "collidergui", docs: "Show collider GUI")]
        public static void RegisterCommands()
        {
            ShowColliders.Instance.ShowGUI();
        }

        public void ShowGUI()
        {
            if (colliderCanvas == null)
                InstantiateCanvas();
            else
            {
                if (colliderCanvas.activeInHierarchy)
                    colliderCanvas.SetActive(false);
                else
                    colliderCanvas.SetActive(true);
            }
        }

        private void InstantiateCanvas()
        {
            var prefab = asset.LoadAsset<GameObject>("ColliderCanvas");
            colliderCanvas = Instantiate(prefab);
            colliderCanvas.AddComponent<ColliderCanvas>();
        }

        public static void ChangeSettings(Color colliderColor, Color triggerColor, bool boxCollider, bool sphereCollider, bool capsuleCollider, bool boxTrigger, bool sphereTrigger, bool capsuleTrigger)
        {
            normalColor = colliderColor;
            ShowColliders.triggerColor = triggerColor;
            RemoveAllGizmos();
            AddAllGizmos(boxCollider, sphereCollider, capsuleCollider, boxTrigger, sphereTrigger, capsuleTrigger);
        }

        private static void AddAllGizmos(bool boxCollider = false, bool sphereCollider = false, bool capsuleCollider = false, bool boxTrigger = false, bool sphereTrigger = false, bool capsuleTrigger = false)
        {
            if (boxCollider || boxTrigger)
                AddGizmos<BoxCollider, BoxColliderGizmo>(boxCollider, boxTrigger);
            if (sphereCollider || sphereTrigger)
                AddGizmos<SphereCollider, SphereColliderGizmo>(sphereCollider, sphereTrigger);
            if (capsuleCollider || capsuleTrigger)
                AddGizmos<CapsuleCollider, CapsuleColliderGizmo>(capsuleCollider, capsuleTrigger);
        }

        private static void AddGizmos<T, T2>(bool enableCollider, bool enableTrigger) where T : UnityEngine.Collider where T2 : BaseGizmo
        {
            var list = FindObjectsOfType<T>();
            if (list != null)
            {
                foreach (var item in list)
                {
                    if (item.name.Validate())
                    {
                        if (enableTrigger && item.isTrigger)
                        {
                            item.gameObject.AddComponent<T2>().LineColor = triggerColor;
                        }
                        if (enableCollider && !item.isTrigger)
                        {
                            item.gameObject.AddComponent<T2>().LineColor = normalColor;
                        }
                    }
                }
            }
        }

        private static void RemoveAllGizmos()
        {
            RemoveGizmos<BoxColliderGizmo>();
            RemoveGizmos<SphereColliderGizmo>();
            RemoveGizmos<CapsuleColliderGizmo>();
        }

        private static void RemoveGizmos<T>() where T : UnityEngine.Object
        {
            var list = FindObjectsOfType<T>();
            if (list != null)
            {
                foreach (var item in list)
                {
                    Destroy(item);
                }
            }
        }

        public void OnModUnload()
        {
            RemoveAllGizmos();
            colliderShader = null;
            asset.Unload(true);
            Debug.Log("ShowColliders has been unloaded!");
            Destroy(colliderCanvas);
            Destroy(gameObject);
        }
    }
}